/*
chimer:

This program will beep the hour in binary on the next minute change.
It's meant to be run by cron at :59, in order to beep on the hour.

Copyright (C) 2005-2008 Paul Marks

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>

// Path to beep
#define BEEP "/usr/bin/beep"

// Some beep arguments
#define FREQ "-f"
#define LEN "-l","95"
#define DELAY "-D","75"
#define NEW "-n"

void microsleep(int64_t sleep_usec);
bool gettimedata(int *hour, int64_t *sleep_usec);

int main(int argc, char *argv[])
{
    // Frequencies for 0 and 1
    const char *freq[] = {"850","1350"};
    int hour;
    int64_t sleep_usec;

    if (argc > 1) 
    {
        printf("chimer-1.1 by Paul Marks\n");
        return 0;
    }

    sleep(3);

    // Wait until the minute changes
    while (gettimedata(&hour, &sleep_usec)) {
        microsleep(sleep_usec);
    }

    // Chime the hour in binary
    execl(BEEP, BEEP,
          FREQ, freq[(hour&8)>>3], LEN, DELAY, NEW,
          FREQ, freq[(hour&4)>>2], LEN, DELAY, NEW,
          FREQ, freq[(hour&2)>>1], LEN, DELAY, NEW,
          FREQ, freq[(hour&1)>>0], LEN,
          (char*) NULL);

    printf("FATAL: Could not execute %s\n", BEEP);
    return -1;
}

// Sleep for some number of microseconds
void microsleep(int64_t sleep_usec)
{
    struct timespec ts;
    ts.tv_sec = sleep_usec / 1000000;
    ts.tv_nsec = (sleep_usec % 1000000) * 1000;
    nanosleep(&ts, NULL);
}

// Place the current hour (1..12) into *hour.
// If we're currently at :00 seconds, return false.
// Otherwise, write half the remaining time into *sleep_usec, and return true.
bool gettimedata(int *hour, int64_t *sleep_usec)
{
    // Get current time (sec, usec)
    struct timeval tv;
    gettimeofday(&tv, NULL);

    // Convert to local time.
    struct tm local_time;
    localtime_r(&tv.tv_sec, &local_time);

    // Save hour (1..12)
    *hour = local_time.tm_hour % 12;
    if (*hour == 0) {
        *hour = 12;
    }

    // If we're already at :00, tell the caller not to sleep.
    if (local_time.tm_sec == 0) {
        return false;
    }

    // Find out how far into the current minute we are.
    const int64_t usec_into_minute = local_time.tm_sec * 1000000 + tv.tv_usec;

    // Calculate half the remaining time to the next minute.
    *sleep_usec = (60000000 - usec_into_minute) / 2;

    // Leap second?  Just sleep for 5ms.
    if (*sleep_usec <= 0) {
      *sleep_usec = 5000;
    }

    // Tell the caller to sleep now.
    return true;
}
